#
# Rules and settings specific to TI's cgtools
#
ASM                    := .sa
OBJ                    := .obj
EXE                    := .out
LIBEXT                 := .a

# Figure out which cgtools must be used
CGTOOLS                 = $(word 2,$(subst -cg, ,$(HALCONARCH)))

# If the target-specific makefile didn't set the SDK_PATH variable,
# attempt to figure it out from the HALCONARCH name.
ifeq ($(SDK_PATH),)
ifeq ($(CGTOOLS),61)
ifeq ($(OS),Windows_NT)
      SDK_PATH    = C:\TI\cgtools_6.1.20
else
      SDK_PATH    = /proj/he/opt/ti/cg6x_6.1
endif
else ifeq ($(CGTOOLS),73)
ifeq ($(OS),Windows_NT)
      SDK_PATH    = C:\TI\ccsv5\tools\compiler\c6000
else
      SDK_PATH    = /proj/he/opt/ti/cg6x_7.3
endif
else ifeq ($(CGTOOLS),74)
ifeq ($(OS),Windows_NT)
      SDK_PATH    = C:\TI\ccsv5\tools\compiler\c6000_7.4.1
else
      SDK_PATH    = /proj/he/opt/ti/cg6x_7.4.11
endif
else
$(error Unknown version of cgtools)
endif
endif

# If 'SDK_PATH' is set, expand PATH so that the compiler can be found.
# Also set C6X_C_DIR correctly.
ifneq ($(SDK_PATH),)
  ifeq ($(OS),Windows_NT)
    PATH               := $(SDK_PATH)\bin;$(PATH)
    export C6X_C_DIR   := $(SDK_PATH)\include;$(SDK_PATH)\lib
  else
    PATH               := $(SDK_PATH)/bin:$(PATH)
    export C6X_C_DIR   := $(SDK_PATH)/include;$(SDK_PATH)/lib
  endif
endif

# As CC, CXX and LD are pre-defined by gmake by default, using ?= so that the
# Makefile can override them won't work.
ifeq ($(origin CC),default)
CC                     := cl6x
endif
ifeq ($(origin CXX),default)
CXX                    := cl6x
endif
ifeq ($(origin LD),default)
LD                     := cl6x $(TARGET_ARCHFLAGS) -z
endif
LXX                    ?= cl6x $(TARGET_ARCHFLAGS) -z
ifeq ($(origin AR),default)
AR                     := ar6x
endif

# Functions to generate appropriate names
asmname                ?= $(1)$(ASM)
objname                ?= $(1)$(OBJ)
outname                ?= $(1)$(EXE)
libname                ?= lib$(1)$(LIBEXT)

# Functions to generate the correct command line switches
adddef                 ?= -D"$(1)"
addipath               ?= -i"$(1)"
addlpath               ?= -i"$(1)"
addlib                 ?= -l$(call libname,$(1))
objout                 ?= -c -fe $(1)
exeout                 ?= -o $(1)
lstatic                ?=
ldynamic               ?=
debug                  ?= -g $(1)
ldebug                 ?= $(1)
profile                ?= $(1)
lprofile               ?= $(1)

# Also look for source files in the 'generated' directory.
vpath %.s               $(GENDIR)
vpath %.S               $(GENDIR)
vpath %.c               $(GENDIR)
vpath %.cpp             $(GENDIR)
vpath %.cc              $(GENDIR)

# Generic rules
$(OBJPAT)$(OBJ): %$(ASM) | $(OBJDIR) ; $(CC) $(CFLAGS) $(call objout,$@) $<

$(OBJPAT)$(OBJ): %.c | $(OBJDIR) ; $(CC) $(CPPFLAGS) $(CFLAGS) $(call objout,$@) $<

$(OBJPAT)$(OBJ): %.cc | $(OBJDIR) ; $(CXX) $(CPPXXFLAGS) $(CXXFLAGS) $(call objout,$@) $<

$(OBJPAT)$(OBJ): %.cpp | $(OBJDIR) ; $(CXX) $(CPPXXFLAGS) $(CXXFLAGS) $(call objout,$@) $<

#
# Generate dependencies. As cl6x cannot write dependency information to stdout,
# only to files, we use an intermediate .ppd file. Since Windows doesn't have
# sed, we can only generate dependencies on Unix systems.
#
ifneq ($(OS), Windows_NT)

$(GENPAT).d: $(GENDIR)$(DIRSEP)%.ppd | $(GENDIR)
	@sed -e 's,^\(.*\)\$(OBJ): ,$(OBJDIR)/\1$(OBJ) $(GENDIR)/\1.d: ,' \
	     -e 's,$(HALCONROOT),\$$\(HALCONROOT\),g' \
	     "$<" >"$@"
	@[ -s "$@" ] || $(RM) "$@"

$(GENPAT).ppd: %.c $(GENERATED) | $(GENDIR)
	@echo "(Re-)generating dependency for $<"
	@cl6x $(CPPFLAGS) -ppd="$@" "$<"

$(GENPAT).d: %.cpp $(GENERATED) | $(GENDIR)
	@echo "(Re-)generating dependency for $<"
	@cl6x $(CPPXXFLAGS) -ppd="$@" "$<"

$(GENPAT).d: %.cc $(GENERATED) | $(GENDIR)
	@echo "(Re-)generating dependency for $<"
	@cl6x $(CPPXXFLAGS) -ppd="$@" "$<"

endif
