#
# Rules and settings specific to gcc and gcc-like compilers
#
ASM                    := .s
OBJ                    := .o
EXE                    :=
LIBEXT                 := .a
DLLEXT                 ?= .so

# On Unix-like systems, shared objects are stored in the library directory
DLLDIR                 := $(LIBDIR)
DLLPAT                 := $(LIBPAT)
DLLPATH                := $(LIBPATH)

# As CC, CXX and LD are pre-defined by gmake by default, using ?= so that the
# Makefile can override them won't work.
ifeq ($(origin CC),default)
CC                     := $(TARGET_PREFIX)gcc
endif
ifeq ($(origin CXX),default)
CXX                    := $(TARGET_PREFIX)g++
endif
ifeq ($(origin LD),default)
LD                     := $(TARGET_PREFIX)gcc
endif
LXX                    ?= $(TARGET_PREFIX)g++

# If 'CROSSROOT' is set, add CROSSROOT/bin to the PATH variable so the
# cross compiler can be found.
ifneq ($(CROSSROOT),)
PATH                   := $(CROSSROOT)$(DIRSEP)bin$(PATHSEP)$(PATH)
endif

# If 'SYSROOT' is set, add it to the command names.
ifneq ($(SYSROOT),)
CC                     += --sysroot $(SYSROOT)
CXX                    += --sysroot $(SYSROOT)
LD                     += --sysroot $(SYSROOT)
LXX                    += --sysroot $(SYSROOT)
endif

# Functions to generate appropriate names. Note that libname and dllname
# are from the perspective of the linker, i.e. the names of the files that
# the linker requires. This makes a difference on Windows, where you link
# against .lib, not the .dll used during runtime.
asmname                ?= $(1)$(ASM)
objname                ?= $(1)$(OBJ)
outname                ?= $(1)$(EXE)
libname                ?= lib$(1)$(LIBEXT)
dllname                ?= lib$(1)$(DLLEXT)

# Functions to generate the correct command line switches
adddef                 ?= -D"$(1)"
addipath               ?= -I"$(1)"
addlpath               ?= -L"$(1)"
addlib                 ?= -l$(1)
objout                 ?= -c -o $(1)
exeout                 ?= -o $(1)
dllout                 ?= -shared -o $(1)
shared                 ?= -fPIC
lstatic                ?= -Wl,-Bstatic
ldynamic               ?= -Wl,-Bdynamic
debug                  ?= -g $(1)
ldebug                 ?= -g $(1)
profile                ?= -pg $(filter-out -fomit-frame-pointer,$(1))
lprofile               ?= -pg $(1)

# Also look for source files in the 'generated' directory.
vpath %.s               $(GENDIR)
vpath %.S               $(GENDIR)
vpath %.c               $(GENDIR)
vpath %.cpp             $(GENDIR)
vpath %.cc              $(GENDIR)

# Generic rules
$(GENPAT)$(ASM): %.S | $(GENDIR) ; $(CC) -E $(AFLAGS) $< >$@

$(GENPAT)$(ASM): %.c | $(GENDIR) ; $(CC) $(CPPFLAGS) $(CFLAGS) -S -o $@ $<

$(GENPAT)$(ASM): %.cc | $(GENDIR) ; $(CC) $(CPPXXFLAGS) $(CXXFLAGS) -S -o $@ $<

$(GENPAT)$(ASM): %.cpp | $(GENDIR) ; $(CC) $(CPPXXFLAGS) $(CXXFLAGS) -S -o $@ $<

$(OBJPAT)$(OBJ): %$(ASM) | $(OBJDIR) ; $(CC) $(AFLAGS) $(call objout,$@) $<

$(OBJPAT)$(OBJ): %.S | $(OBJDIR) ; $(CC) $(AFLAGS) $(call objout,$@) $<

$(OBJPAT)$(OBJ): %.c | $(OBJDIR) ; $(CCCMD)

$(OBJPAT)$(OBJ): %.cc | $(OBJDIR) ; $(CXXCMD)

$(OBJPAT)$(OBJ): %.cpp | $(OBJDIR) ; $(CXXCMD)

# Generic rules for Objective-C and Objective-C++ files (for OS X)
$(OBJPAT)$(OBJ): %.m | $(OBJDIR) ; $(CCCMD)

$(OBJPAT)$(OBJ): %.mm | $(OBJDIR) ; $(CXXCMD)


# Special rule to generate a preprocessed file. Don't rely on this rule just
# yet; it is subject to change.
$(GENPAT).i:     %.c | $(GENDIR)
	$(CC) -E $(CPPFLAGS) $(CFLAGS) $(call objout,$@) $<

$(GENPAT).i:     %.cc | $(GENDIR)
	$(CXX) -E $(CPPXXFLAGS) $(CXXFLAGS) $(call objout,$@) $<

$(GENPAT).i:     %.cpp | $(GENDIR)
	$(CXX) -E $(CPPXXFLAGS) $(CXXFLAGS) $(call objout,$@) $<

# Create dependency file from C source file. Note that the funky
# sed is necessary in order to have the dependency file itself be updated
# automatically. Also note that the C source file may be in the current
# directory or $(GENDIR).
$(GENPAT).d: $(GENPATH)%.c | $(GENDIR)
	@echo "(Re-)generating dependency for $<"
	@$(CC) -MM $(CPPFLAGS) $(CFLAGS) "$<" \
	| sed -e 's,^\(.*\)\$(OBJ): ,$(OBJDIR)/\1$(OBJ) $(GENDIR)/\1.d: ,' \
	      -e 's,$(HALCONROOT),\$$\(HALCONROOT\),g' \
	  > "$@"; [ -s "$@" ] || $(RM) "$@"

$(GENPAT).d: %.c | $(GENDIR)
	@echo "(Re-)generating dependency for $<"
	@$(CC) -MM $(CPPFLAGS) $(CFLAGS) "$<" \
	| sed -e 's,^\(.*\)\$(OBJ): ,$(OBJDIR)/\1$(OBJ) $(GENDIR)/\1.d: ,' \
	      -e 's,$(HALCONROOT),\$$\(HALCONROOT\),g' \
	  > "$@"; [ -s "$@" ] || $(RM) "$@"

$(GENPAT).d: $(GENPATH)%.cpp | $(GENDIR)
	@echo "(Re-)generating dependency for $<"
	@$(CXX) -MM $(CPPXXFLAGS) $(CXXFLAGS) "$<" \
	| sed -e 's,^\(.*\)\$(OBJ): ,$(OBJDIR)/\1$(OBJ) $(GENDIR)/\1.d: ,' \
	      -e 's,$(HALCONROOT),\$$\(HALCONROOT\),g' \
	  > "$@"; [ -s "$@" ] || $(RM) "$@"

$(GENPAT).d: %.cpp | $(GENDIR)
	@echo "(Re-)generating dependency for $<"
	@$(CXX) -MM $(CPPXXFLAGS) $(CXXFLAGS) "$<" \
	| sed -e 's,^\(.*\)\$(OBJ): ,$(OBJDIR)/\1$(OBJ) $(GENDIR)/\1.d: ,' \
	      -e 's,$(HALCONROOT),\$$\(HALCONROOT\),g' \
	  > "$@"; [ -s "$@" ] || $(RM) "$@"

$(GENPAT).d: $(GENPATH)%.cc | $(GENDIR)
	@echo "(Re-)generating dependency for $<"
	@$(CXX) -MM $(CPPXXFLAGS) $(CXXFLAGS) "$<" \
	| sed -e 's,^\(.*\)\$(OBJ): ,$(OBJDIR)/\1$(OBJ) $(GENDIR)/\1.d: ,' \
	      -e 's,$(HALCONROOT),\$$\(HALCONROOT\),g' \
	  > "$@"; [ -s "$@" ] || $(RM) "$@"

$(GENPAT).d: %.cc | $(GENDIR)
	@echo "(Re-)generating dependency for $<"
	@$(CXX) -MM $(CPPXXFLAGS) $(CXXFLAGS) "$<" \
	| sed -e 's,^\(.*\)\$(OBJ): ,$(OBJDIR)/\1$(OBJ) $(GENDIR)/\1.d: ,' \
	      -e 's,$(HALCONROOT),\$$\(HALCONROOT\),g' \
	  > "$@"; [ -s "$@" ] || $(RM) "$@"
