#
# Settings and rules to use for projects using HALCON
#
# First, make sure any spaces in HALCONROOT are escaped to avoid confusing
# (GNU)make. Unfortunately, Windows doesn't know about escaping spaces,
# so we need to unescape paths before passing them to the Windows command
# line.
HALCON_ROOT            := $(call escape_path,$(HALCONROOT))
HALCON_HDRDIR           = $(HALCON_ROOT)$(DIRSEP)include
# HALCON 10 has the old C++ interface.
ifneq ($(filter x-yes,x-$(HALCON10)),)
HALCON_HXXDIR           = $(HALCON_HDRDIR)$(DIRSEP)cpp
else
HALCON_HXXDIR           = $(HALCON_HDRDIR)$(DIRSEP)halconcpp
endif

# On Windows, determine the Visual Studio version (MSDEV_VERSION) if not
# already set.
ifneq ($(or $(filter win32,$(TARGET_OS)),$(filter win64,$(TARGET_OS)),\
            $(filter wince%,$(TARGET_OS))), )
  # Attempt to figure out which version of Visual Studio we're using (we assume
  # a Windows target will *always* be compiled with Visual Studio)
  ifndef MSDEV_VERSION
    EMPTY = 
    BLANK = $(EMPTY) $(EMPTY)
    ifdef VisualStudioVersion
      # Visual Studio 2012 and later set the VisualStudioVersion environment
      # variable
      MSDEV_VERSION = $(firstword $(subst ., $(BLANK), $(VisualStudioVersion)))
    else ifdef WIN_TITLE_VS
      # Visual Studio 2010 and earlier set WIN_TITLE_VS
      MSDEV_VERSION = $(lastword $(subst 20, , $(WIN_TITLE_VS)))
    else ifdef VSINSTALLDIR
      # For really old Visual Studio build shells, get the version by
      # filtering VSINSTALLDIR
      MSDEV_VERSION = $(filter 8 9 10 11, $(subst .,$(BLANK),$(VSINSTALLDIR)))
    endif

    ifeq ($(EMPTY),$(MSDEV_VERSION))
      # If none of the methods above to determine the MSDEV_VERSION succeeded,
      # set the default runtime version to 10.
      MSDEV_VERSION = 10
      $(warning  +++ WARNING: no VisualStudioVersion specified; defaulting MSDEV_VERSION to $(MSDEV_VERSION))
    else
      $(info +++ using MSDEV_VERSION=$(MSDEV_VERSION))
    endif
  endif
endif

# Set HALCONARCH_EXT to the 'extended' HALCONARCH. This will include the
# compiler (and runtime environment for Windows) if COMP_ARCH is set.
ifneq ($(and $(filter yes,$(COMP_ARCH)),$(COMP)),)
  ifneq ($(or $(filter win32,$(TARGET_OS)),$(filter win64,$(TARGET_OS)),\
              $(filter wince%,$(TARGET_OS))), )
    ifeq ($(FORCE_CFLAGS_MD), yes)
      HALCONARCH_EXT := $(HALCONARCH)-$(COMP)$(MSDEV_VERSION)md
      # In our template, the runtime is selected via 'RUNTIME', not
      # FORCE_CFLAGS_MD
      RUNTIME=dynamic
    else
      HALCONARCH_EXT := $(HALCONARCH)-$(COMP)$(MSDEV_VERSION)
    endif
  else
    HALCONARCH_EXT := $(HALCONARCH)-$(COMP)
  endif
else
  HALCONARCH_EXT := $(HALCONARCH)
endif

ifeq ($(STATE),debugXXX)
HALCON_BINDIR           = $(HALCON_ROOT)$(DIRSEP)bind$(DIRSEP)$(HALCONARCH_EXT)
HALCON_LIBDIR           = $(HALCON_ROOT)$(DIRSEP)libd$(DIRSEP)$(HALCONARCH_EXT)
else ifeq ($(STATE),tuning)
HALCON_BINDIR           = $(HALCON_ROOT)$(DIRSEP)bint$(DIRSEP)$(HALCONARCH_EXT)
HALCON_LIBDIR           = $(HALCON_ROOT)$(DIRSEP)libt$(DIRSEP)$(HALCONARCH_EXT)
else
HALCON_BINDIR           = $(HALCON_ROOT)$(DIRSEP)bin$(DIRSEP)$(HALCONARCH_EXT)
HALCON_LIBDIR           = $(HALCON_ROOT)$(DIRSEP)lib$(DIRSEP)$(HALCONARCH_EXT)
endif
HALCON_CPPFLAGS         = $(call addipath,$(call unescape_path,$(HALCON_HDRDIR)))
HALCON_CPPXXFLAGS       = $(call addipath,$(call unescape_path,$(HALCON_HDRDIR))) \
                          $(call addipath,$(call unescape_path,$(HALCON_HXXDIR)))
HALCON_HDEVCPPCPPFLAGS  = $(HALCON_CPPXXFLAGS) \
                          $(call addipath,$(call unescape_path,$(HALCON_HDRDIR)$(DIRSEP)hdevengine))
HALCON_LDFLAGS          = $(call addlpath,$(call unescape_path,$(HALCON_LIBDIR)))
HALCON_LXXFLAGS         = $(call addlpath,$(call unescape_path,$(HALCON_LIBDIR)))

# Select between parallel and sequential HALCON
ifeq ($(PARALLEL),no)
HALCON_PREFIX          := seq
HVERSIONPATH           := seq
else
HVERSIONPATH           := par
endif

# Select between HALCON XL and normal HALCON
ifeq ($(XL),yes)
HALCON_SUFFIX           = xl
HALCON_CPPFLAGS        += $(call adddef,HC_LARGE_IMAGES)
HALCON_CPPXXFLAGS      += $(call adddef,HC_LARGE_IMAGES)
HVERSIONPATH           := $(HVERSIONPATH)_xl
endif

# For HALCON 10 and earlier, the static libraries are "unofficial" only and
# are named just like the shared library (but with a different system-dependent
# file type suffix). For HALCON 11 and later, static libraries are postfixed
# with "_static". While it would be nice to be able to automatically figure
# out the version of HALCON used, this cannot really be done portably for
# Linux and Windows, so we rely on the user setting HALCON10 to "yes" if
# HALCON 10 (or earlier) is to be used.
ifneq ($(and $(filter yes,$(STATIC)),$(filter-out x-yes,x-$(HALCON10))),)
HALCON_SUFFIX          := $(HALCON_SUFFIX)_static
HVERSIONPATH           := $(HVERSIONPATH)_static
endif

# Override the generated and object directories so that we can compile various
# versions of HALCON (non-XL, XL, parallel, sequential, static, etc.) side by
# side. The binary and library directories are not renamed, as the expectation
# is that the targets (binaries and libraries) will have the
# 'xl' suffix in their name.
GENDIR                 := $(GENPATH)$(HVERSIONPATH)
GENPAT                 := $(GENDIR)$(ESCDIRSEP)%
GENPATH                := $(GENDIR)$(DIRSEP)
OBJDIR                 := $(OBJPATH)$(HVERSIONPATH)
OBJPAT                 := $(OBJDIR)$(ESCDIRSEP)%
OBJPATH                := $(OBJDIR)$(DIRSEP)

# Base names of HALCON libraries (i.e. {,seq}halcon{,xl})
HALCON_HLIBNAME         = $(HALCON_PREFIX)halcon$(HALCON_SUFFIX)
HALCON_CINTNAME         = $(HALCON_PREFIX)halconc$(HALCON_SUFFIX)
HALCON_CPPINTNAME       = $(HALCON_PREFIX)halconcpp$(HALCON_SUFFIX)
HALCON_HDEVCPPNAME      = $(HALCON_PREFIX)hdevenginecpp$(HALCON_SUFFIX)

# Full path to the libraries, for use in dependencies
ifeq ($(STATIC),yes)
HALCON_HLIBLIB          = $(HALCON_LIBDIR)$(DIRSEP)$(call libname,$(HALCON_HLIBNAME))
HALCON_CINTLIB          = $(HALCON_LIBDIR)$(DIRSEP)$(call libname,$(HALCON_CINTNAME))
HALCON_CPPINTLIB        = $(HALCON_LIBDIR)$(DIRSEP)$(call libname,$(HALCON_CPPINTNAME))
HALCON_HDEVCPPLIB       = $(HALCON_LIBDIR)$(DIRSEP)$(call libname,$(HALCON_HDEVCPPNAME))
else
HALCON_HLIBLIB          = $(HALCON_LIBDIR)$(DIRSEP)$(call dllname,$(HALCON_HLIBNAME))
HALCON_CINTLIB          = $(HALCON_LIBDIR)$(DIRSEP)$(call dllname,$(HALCON_CINTNAME))
HALCON_CPPINTLIB        = $(HALCON_LIBDIR)$(DIRSEP)$(call dllname,$(HALCON_CPPINTNAME))
HALCON_HDEVCPPLIB       = $(HALCON_LIBDIR)$(DIRSEP)$(call dllname,$(HALCON_HDEVCPPNAME))
endif

# NOTE: HDevEngine needs both the C and C++ language interface libraries.
ifeq ($(STATIC),yes)
HALCON_LDLIBS           = $(call lstatic)\
                          $(call addlib,$(HALCON_CINTNAME))\
                          $(call addlib,$(HALCON_HLIBNAME))\
                          $(call ldynamic)
HALCON_LXXLIBS          = $(call lstatic)\
                          $(call addlib,$(HALCON_CPPINTNAME))\
                          $(call addlib,$(HALCON_HLIBNAME))\
                          $(call ldynamic)
HALCON_HDEVCPPLIBS      = $(call lstatic)\
                          $(call addlib,$(HALCON_HDEVCPPNAME))\
                          $(call addlib,$(HALCON_CINTNAME))\
                          $(call addlib,$(HALCON_CPPINTNAME))\
                          $(call addlib,$(HALCON_HLIBNAME))\
                          $(call ldynamic)
else
HALCON_LDLIBS           = $(call addlib,$(HALCON_CINTNAME))\
                          $(call addlib,$(HALCON_HLIBNAME))
HALCON_LXXLIBS          = $(call addlib,$(HALCON_CPPINTNAME))\
                          $(call addlib,$(HALCON_HLIBNAME))
HALCON_HDEVCPPLIBS      = $(call addlib,$(HALCON_HDEVCPPNAME))\
                          $(call addlib,$(HALCON_CINTNAME))\
                          $(call addlib,$(HALCON_CPPINTNAME))\
                          $(call addlib,$(HALCON_HLIBNAME))
endif

#
# For MacOS, use frameworks instead of libraries (when not using static
# libraries). We simply override the existing definitions above so as not to
# clutter the makefile with conditionals to the point of total unreadability.
#
# Note that as of HALCON 12, code exported from HDevelop must also link against
# the CoreFoundation framework explicitly.
ifeq ($(TARGET_OS),macosx)
ifneq ($(STATIC),yes)
HALCON_CPPFLAGS         = -F$(HALCON_LIBDIR)
HALCON_CPPXXFLAGS       = -F$(HALCON_LIBDIR)
HALCON_HDEVCPPCPPFLAGS  = -F$(HALCON_LIBDIR)
HALCON_LDFLAGS          = -F$(HALCON_LIBDIR)
HALCON_LXXFLAGS         = -F$(HALCON_LIBDIR)

HALCON_HLIBNAME         = HALCON$(HALCON_SUFFIX)
HALCON_CINTNAME         = HALCONC$(HALCON_SUFFIX)
HALCON_CPPINTNAME       = HALCONCpp$(HALCON_SUFFIX)
HALCON_HDEVCPPNAME      = HDevEngineCpp$(HALCON_SUFFIX)

HALCON_HLIBLIB          = $(HALCON_LIBDIR)$(DIRSEP)$(HALCON_HLIBNAME).framework
HALCON_CINTLIB          = $(HALCON_LIBDIR)$(DIRSEP)$(HALCON_CINTNAME).framework
HALCON_CPPINTLIB        = $(HALCON_LIBDIR)$(DIRSEP)$(HALCON_CPPINTNAME).framework
HALCON_HDEVCPPLIB       = $(HALCON_LIBDIR)$(DIRSEP)$(HALCON_HDEVCPPNAME).framework

HALCON_LDLIBS           = -framework $(HALCON_CINTNAME)\
                          -framework $(HALCON_HLIBNAME)\
                          -framework CoreFoundation
HALCON_LXXLIBS          = -framework $(HALCON_CPPINTNAME)\
                          -framework $(HALCON_HLIBNAME)\
                          -framework CoreFoundation
HALCON_HDEVCPPLIBS      = -framework $(HALCON_HDEVCPPNAME)\
                          -framework $(HALCON_CPPINTNAME)\
                          -framework $(HALCON_HLIBNAME)\
                          -framework CoreFoundation
endif
endif

# HDevelop to use for code export.
# TODO: specify the exact hdevelop binary to use and don't rely on the PATH
# to select the correct one.
ifneq ($(TARGET_OS),macosx)
HALCON_HDEVELOP         = hdevelop
else
HALCON_HDEVELOP         = $(HALCON_ROOT)/bin/$(HALCONARCH_EXT)/hdevelop.app/Contents/MacOS/hdevelop
endif

#
# Generic rules for HALCON code export. Note that these files are *not*
# created in the GENDIR directory because they are independent of the
# target architecture.
#
%.c: %.dev; $(HALCON_HDEVELOP) -convert $< $@

%.c: %.hdev; $(HALCON_HDEVELOP) -convert $< $@

%.cpp: %.dev; $(HALCON_HDEVELOP) -convert $< $@

%.cpp: %.hdev; $(HALCON_HDEVELOP) -convert $< $@
